@@ -1,11 +1,11 @@
 #' @title Coupled leaf gas exchange model
 #' @description A coupled photosynthesis - stomatal conductance model, based on the Farquhar model of photosynthesis, and a Ball-Berry type model of stomatatal conductance. Includes options for temperature sensitivity of photosynthetic parameters, dark respiration (optionally calculated from leaf temperature), and mesophyll conductance. 
-#' @param VPD Vapour pressure deficit (kPa)
+#' @param VPD Vapour pressure deficit (kPa) (not needed when RH provided)
 #' @param Ca Atmospheric CO2 concentration (ppm)
 #' @param PPFD Photosynthetic photon flux density ('PAR') (mu mol m-2 s-1)
 #' @param Tleaf Leaf temperature (degrees C)
 #' @param Patm Atmospheric pressure (kPa) (but see warning below!)
-#' @param RH Relative humidity (in \%)
+#' @param RH Relative humidity (in \%) (not needed when VPD provided)
 #' @param gsmodel One of BBOpti (Medlyn et al. 2011), BBLeuning (Leuning 1995), or BallBerry (Ball et al. 1987)
 #' @param g0,g1 Parameters of Ball-Berry type stomatal conductance models.
 #' @param gk Optional, exponent of VPD in gs model (Duursma et al. 2013)
@@ -17,6 +17,7 @@
 #' @param Jmax Maximum rate of electron transport at 25 degrees C (mu mol m-2 s-1)
 #' @param Vcmax Maximum carboxylation rate at 25 degrees C (mu mol m-2 s-1)
 #' @param gmeso Mesophyll conductance (mol m-2 s-1). If not NULL (the default), Vcmax and Jmax are chloroplastic rates.
+#' @param TPU Triose-phosphate utilization rate (mu mol m-2 s-1); optional.
 #' @param Rd Dark respiration rate (mu mol m-2 s-1), optional (if not provided, calculated from Tleaf, Rd0, Q10 and TrefR)
 #' @param Rd0 Dark respiration rata at reference temperature (\code{TrefR})
 #' @param Q10 Temperature sensitivity of Rd.
@@ -31,33 +32,31 @@
 #' @param whichA Which assimilation rate does gs respond to? 
 #' @param \dots Further arguments passed to \code{Photosyn}
 #' @seealso \code{\link{FARAO}}, \code{\link{fitaci}}, \code{\link{AciC4}}
-#' @details The coupled photosynthesis - stomatal conductance model finds the intersection between the supply of CO2 by diffusion, and the demand for CO2 by photosynthesis. See Farquhar and Sharkey (1982) for basic description of this type of model. 
+#' @details The coupled photosynthesis - stomatal conductance model finds the intersection between the supply of CO2 by diffusion, and the demand for CO2 by photosynthesis. See Farquhar and Sharkey (1982) for basic description of this type of model, Duursma (2015) for more details on the implementation in the \code{plantecophys} package, and Duursma et al. (2014) for an example application (that uses this implementation).
 #' 
-#' The model of Farquhar et al. (1980) is used to estimate the dependence of photosynthesis rate on Ci.
-#'  
-#'  The temperature response of photosynthetic parameters, including Vcmax, Jmax, Gammastar, and Km follow Medlyn et al. 2002. 
+#' The model of Farquhar et al. (1980) is used to estimate the dependence of leaf net photosynthesis rate (ALEAF) on intercellular CO2 concentration (Ci).   The temperature response of photosynthetic parameters, including Vcmax, Jmax, Gammastar, and Km follow Medlyn et al. (2002). 
 #'  
 #'  At the moment, two stomatal conductance models are implemented, both are Ball-Berry type models. The 'BBOpti' model is a slightly more general form of the model of Medlyn et al. 2011 (see Duursma et al. 2013). It is given by (in notation of the parameters and output variables of \code{Photosyn}),
 #'  
-#'  \deqn{GS = G0 + 1.6*(1 + G1/D^(1-GK))*ALEAF/CA}
+#'  \deqn{GS = g0 + 1.6*(1 + g1/D^(1-gk))*ALEAF/CA}
 #'  
-#'  where GK = 0.5 if stomata behave optimally (cf. Medlyn et al. 2011).
+#'  where gk = 0.5 if stomata behave optimally (cf. Medlyn et al. 2011).
 #'  
 #'  The 'BBLeuning' model is that of Leuning (1995). It is given by,
 #'  
-#'  \deqn{GS = G0 + g1*ALEAF/(Ca * (1 + VPD/D0))}
+#'  \deqn{GS = g0 + g1*ALEAF/(Ca * (1 + VPD/D0))}
 #'  
 #'  Note that this model also uses the g1 parameter, but it needs to be set to a much higher value to be comparable in magnitude to the BBOpti model.
 #'  
 #'  The 'BallBerry' model is that of Ball et al. (1987). It is given by,
 #'  
-#'  \deqn{GS = G0 + g1*RH*ALEAF/Ca}
+#'  \deqn{GS = g0 + g1*RH*ALEAF/Ca}
 #'  
 #'  Where RH is relative humidity.
 #'  
-#'  For the full numerical solution to the Cowan-Farquhar optimization, use the \code{\link{FARAO}} function.
+#'  For the full numerical solution to the Cowan-Farquhar optimization, use the \code{\link{FARAO}} function (which was used in Medlyn et al. 2011 for comparison to the approximation there presented).
 #'  
-#'  If the mesophyll conductance is provided, it is assumed that Vcmax and Jmax are the chloroplastic rates, and leaf photosynthesis is calculated following Ethier and Livingston (2004).
+#'  If the mesophyll conductance \code{gmeso} is provided, it is assumed that Vcmax and Jmax are the chloroplastic rates, and leaf photosynthesis is calculated following Ethier and Livingston (2004).
 #'  
 #'  If Ci is provided as an input, this function calculates an A-Ci curve. Otherwise, Ci is calculated from the intersection between the 'supply' and 'demand', where 'demand' is given by the Farquhar model of photosynthesis (A=f(Ci)), and supply by the stomatal conductance. The latter is, by default, estimated using the stomatal conductance model of Medlyn et al. (2011), but two other models are provided as well (Ball-Berry and Leuning, see \code{gsmodel} argument). Otherwise, stomatal conductance may be directly provided via the \code{GS} argument. 
 #'  
@@ -72,6 +71,10 @@
 #' 
 #' @references 
 #' Duursma, R.A., Payton, P., Bange, M.P., Broughton, K.J., Smith, R.A., Medlyn, B.E., Tissue, D. T., 2013,  Near-optimal response of instantaneous transpiration efficiency to vapour pressure deficit, temperature and [CO2] in cotton (Gossypium hirsutum L.). Agricultural and Forest Meteorology 168 : 168 - 176.
+#' 
+#' Duursma, R.A., Barton, C.V.M., Lin, Y.-S., Medlyn, B.E., Eamus, D., Tissue, D.T., Ellsworth, D.S., McMurtrie, R.E., 2014. The peaked response of transpiration rate to vapour pressure deficit in field conditions can be explained by the temperature optimum of photosynthesis. Agricultural and Forest Meteorology 189 - 190, 2-10. doi:10.1016/j.agrformet.2013.12.007
+#'
+#' Duursma, R.A., 2015. Plantecophys - An R Package for Analysing and Modelling Leaf Gas Exchange Data. PLoS ONE 10, e0143346. doi:10.1371/journal.pone.0143346
 #'
 #'Ethier, G. and N. Livingston. 2004. On the need to incorporate sensitivity to CO2 transfer conductance into the Farquhar von Caemmerer Berry leaf photosynthesis model. Plant, Cell & Environment. 27:137-153.
 #'
@@ -156,6 +159,7 @@ Photosyn <- function(VPD=1.5,
                      Jmax=100, 
                      Vcmax=50, 
                      gmeso=NULL,
+                     TPU=1000,
                      
                      Rd0 = 0.92,
                      Q10 = 1.92,
@@ -191,12 +195,6 @@ Photosyn <- function(VPD=1.5,
   Rgas <- .Rgas()
   GCtoGW <- 1.57     # conversion from conductance to CO2 to H2O
   
-  #---- Functions
-  # Non-rectangular hyperbola
-  Jfun <- function(PPFD, alpha, Jmax, theta){
-    (alpha*PPFD + Jmax - 
-       sqrt((alpha*PPFD + Jmax)^2 - 4*alpha*theta*PPFD*Jmax))/(2*theta)
-  }
   
   #---- Do all calculations that can be vectorized
   
@@ -221,15 +219,15 @@ Photosyn <- function(VPD=1.5,
     Jmax <- Jmax * TJmax(Tleaf, EaJ, delsJ, EdVJ)
   }
   
-  #--- Stop here if only the parameters are required
-  if(returnParsOnly){
-    return(list(Vcmax=Vcmax, Jmax=Jmax, Km=Km, GammaStar=GammaStar))
-  }
-  
   # Electron transport rate
   J <- Jfun(PPFD, alpha, Jmax, theta)
   VJ <- J/4
   
+  #--- Stop here if only the parameters are required
+  if(returnParsOnly){
+    return(list(Vcmax=Vcmax, Jmax=Jmax, Km=Km, GammaStar=GammaStar, VJ=VJ))
+  }
+  
   # Medlyn et al. 2011 model gs/A. NOTE: 1.6 not here because we need GCO2!
   if(gsmodel == "BBOpti"){
     vpduse <- VPD
@@ -337,14 +335,18 @@ Photosyn <- function(VPD=1.5,
         # Ci provided (A-Ci function mode)
         CIJ <- Ci
         
-        CIJ[CIJ < GammaStar] <- GammaStar[CIJ < GammaStar]
+        if(length(GammaStar) > 1){
+          CIJ[CIJ < GammaStar] <- GammaStar[CIJ < GammaStar]
+        } else {
+          CIJ[CIJ < GammaStar] <- GammaStar
+        }
         
         CIC <- Ci
         
       }
   
     # Photosynthetic rates, without or with mesophyll limitation
-    if(is.null(gmeso)){
+    if(is.null(gmeso) || gmeso < 0){
       # Get photosynthetic rate  
       Ac <- Vcmax*(CIC - GammaStar)/(CIC + Km)
       Aj <- VJ * (CIJ - GammaStar)/(CIJ + 2*GammaStar)
@@ -364,7 +366,8 @@ Photosyn <- function(VPD=1.5,
       Aj <- Aj + Rd
       
     }
-      
+
+    
       # When below light-compensation points, assume Ci=Ca.
       if(!inputCi){
         lesslcp <- vector("logical", length(Aj))
@@ -381,10 +384,24 @@ Photosyn <- function(VPD=1.5,
         Ci <- ifelse(Aj < Ac, CIJ, CIC)
       }
   }
-  
+
+    # Limitation by triose-phosphate utilization
+    Ap <- 3*TPU   
+    
     # Hyperbolic minimum.
     hmshape <- 0.9999
-    Am <- (Ac+Aj - sqrt((Ac+Aj)^2-4*hmshape*Ac*Aj))/(2*hmshape) - Rd
+    Am <- (Ac+Aj - sqrt((Ac+Aj)^2-4*hmshape*Ac*Aj))/(2*hmshape)
+    
+    # Another hyperbolic minimum with the transition to TPU
+    #browser()
+    tpulim <- any(Ap < Am)
+    if(!is.na(tpulim) && tpulim){
+      hmshape <- 1 - 1E-07
+      Am <- (Am+Ap - sqrt((Am+Ap)^2-4*hmshape*Am*Ap))/(2*hmshape)
+    }
+    
+    # Net photosynthesis
+    Am <- Am - Rd
   
     # Calculate conductance to CO2
     if(!inputCi && !inputGS){
@@ -398,6 +415,9 @@ Photosyn <- function(VPD=1.5,
       if(whichA == "Ac")GS <- (Ac-Rd)/(Ca - Ci)
     }
 
+    # Extra step here; GS can be negative
+    GS[GS < g0] <- g0
+    
     # Output conductance to H2O
     if(!inputGS){
       GS <- GS*GCtoGW
@@ -418,6 +438,7 @@ Photosyn <- function(VPD=1.5,
                       ELEAF=E,
                       Ac=Ac,
                       Aj=Aj,
+                      Ap=Ap,
                       Rd=Rd,
                       VPD=VPD,
                       Tleaf=Tleaf,
@@ -435,5 +456,15 @@ Aci <- function(Ci,...)Photosyn(Ci=Ci,...)
 
 
 
+# Non-rectangular hyperbola
+Jfun <- function(PPFD, alpha, Jmax, theta){
+  (alpha*PPFD + Jmax - 
+     sqrt((alpha*PPFD + Jmax)^2 - 4*alpha*theta*PPFD*Jmax))/(2*theta)
+}
+
+inverseJfun <- function(PPFD, alpha, J, theta){
+  J*(J*theta - alpha*PPFD)/(J - alpha*PPFD)
+}
+
 
 
